@@ -1,7 +1,7 @@
 /*
  * GRSPW2 SpaceWire Kernel Library Interface
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,10 @@
 #ifndef __GRSPW2_LIB_H__
 #define __GRSPW2_LIB_H__
 
+#ifndef __KERNEL__
+#include <stdint.h>
+#endif
+
 struct grspw_pkt;
 
 /* Maximum number of GRSPW devices supported by driver */
@@ -141,9 +145,15 @@ struct grspw_pkt {
 	unsigned char reserved;	/* Reserved, must be zero */
 	unsigned char hlen;	/* Length of Header Buffer (only TX) */
 	unsigned int dlen;	/* Length of Data Buffer */
-	void *data;	/* 4-byte or byte aligned depends on HW */
-	void *hdr;	/* 4-byte or byte aligned depends on HW (only TX) */
+#ifdef __KERNEL__
+	u32 data;		/* 4-byte or byte aligned address depends on HW */
+	u32 hdr;		/* 4-byte or byte aligned addressdepends on HW (only TX) */
+#else
+	uint32_t data;		/* 4-byte or byte aligned address depends on HW */
+	uint32_t hdr;		/* 4-byte or byte aligned address depends on HW (only TX) */
+#endif
 };
+#define GRSPW_PKT_DMAADDR_INVALID 0xFFFFFFFFUL /* indicates no data */
 
 /* GRSPW SpaceWire Packet List */
 struct grspw_list {
