@@ -3,7 +3,7 @@
  * Memory MAPed buffers. For example SpW0 buffers can be used
  * zero-copy to SpW1.
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -53,6 +53,7 @@ struct maplib_mmap_info {
 
 
 /*** KERNEL INTRFACE ***/
+#ifdef __KERNEL__
 
 /* Driver that uses the library */
 struct maplib_drv {
@@ -76,13 +77,16 @@ extern int maplib_drv_reg(int idx, struct maplib_drv *drv);
 /* Unregister Driver */
 extern void maplib_drv_unreg(int idx, struct maplib_drv *drv);
 
+/* Basic check for user-drivers that MAP device is valid */
+extern int maplib_drv_chkvalid(int idx);
+
 /* Translate previously MMAPed User Space Virtual Address into Kernel Address
  * Return -1 if invalid address
  *
  * Check that the complete user/kernel address is mapped and linear
  * into kernel space (and hence user space). Set LEN=0 to avoid check.
  */
-extern int maplib_lookup_kern(int idx, void *usr, void **kern, void **hw, int len);
+extern int maplib_lookup_kern(int idx, void *usr, void **kern, phys_addr_t *hw, int len);
 
 /* Translate previously MMAPed Kernel Virtual Address into UserSpace Address
  * Return -1 if invalid address
@@ -90,6 +94,7 @@ extern int maplib_lookup_kern(int idx, void *usr, void **kern, void **hw, int le
  * Check that the complete user/kernel address is mapped and linear
  * into kernel space (and hence user space). Set LEN=0 to avoid check.
  */
-extern int maplib_lookup_usr(int idx, void *kern, void *hw, void **usr, int len);
+extern int maplib_lookup_usr(int idx, void *kern, phys_addr_t hw, void **usr, int len);
+#endif
 
 #endif
