@@ -1,7 +1,7 @@
 /*
  * Device Memory MAP Library help functions for GRLIB Device Drivers
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -42,6 +42,9 @@ MODULE_LICENSE ("GPL");
 	#define MAPLIB_DBG(doCode)
 #endif
 
+/* this interface was not ported because it was unused going from 3.10 to 5.10 kernel */
+#define MAPLIB_KERN_TO_USR_LOOKUP 0
+
 #define MAPLIB_MAX 1
 
 #if (MAPLIB_MAX > MAPLIB_DEVCNT)
@@ -78,10 +81,12 @@ struct maplib_priv {
 	unsigned long uend;
 	struct vm_area_struct *user_vma;
 
+#if MAPLIB_KERN_TO_USR_LOOKUP
 	/* Kernel Address to Block number Translation Table.
 	 * MSB 8-bit.
 	 */
 	short *kern_tab[256];
+#endif
 
 	/* All drivers attached */
 	struct maplib_drv *drvs;
@@ -153,6 +158,17 @@ void maplib_drv_unreg(int idx, struct maplib_drv *drv)
 }
 EXPORT_SYMBOL(maplib_drv_unreg);
 
+/* Unregister Driver */
+int maplib_drv_chkvalid(int idx)
+{
+	if (idx < 0 || idx >= maplib_count)
+		return -1;
+	if (maplib_privs == NULL)
+		return -1;
+	return 0; /* MAP index exists */
+}
+EXPORT_SYMBOL(maplib_drv_chkvalid);
+
 #if 0
 void maplib_tab_print(struct maplib_priv *priv, short **usrtab)
 {
@@ -180,12 +196,13 @@ void maplib_tab_print(struct maplib_priv *priv, short **usrtab)
 /* Translate previously MMAPed User Space Virtual Address into Kernel Address
  * Return -1 if invalid address
  */
-int maplib_lookup_kern(int idx, void *usr, void **kern, void **hw, int len)
+int maplib_lookup_kern(int idx, void *usr, void **kern, phys_addr_t *hw, int len)
 {
 	struct maplib_priv *priv = &maplib_privs[idx];
 	unsigned long uadr = (unsigned long)usr;
-	int ofs, blkno, blkno_end;
+	int blkno, blkno_end;
 	struct maplib_blk *blk;
+	unsigned long ofs;
 
 	if ( (uadr < priv->ustart) || (uadr >= priv->uend) )
 		return -1;
@@ -200,16 +217,17 @@ int maplib_lookup_kern(int idx, void *usr, void **kern, void **hw, int len)
 	if ( kern )
 		*kern = (void *)((unsigned long)blk->start + ofs);
 	if ( hw )
-		*hw = (void *)(virt_to_phys((unsigned long)blk->start + ofs));
+		*hw = virt_to_phys(blk->start + ofs);
 
 	return 0;
 }
 EXPORT_SYMBOL(maplib_lookup_kern);
 
+#if MAPLIB_KERN_TO_USR_LOOKUP
 /* Translate previously MMAPed Kernel Virtual Address into UserSpace Address
  * Return -1 if invalid address
  */
-int maplib_lookup_usr(int idx, void *kern, void *hw, void **usr, int len)
+int maplib_lookup_usr(int idx, void *kern, phys_addr_t hw, void **usr, int len)
 {
 	int msb = (unsigned int)kern >> 24;
 	int blkidx = ((unsigned int)kern & 0x00FFFFFF) >> BLK_SHIFT;
@@ -232,7 +250,6 @@ int maplib_lookup_usr(int idx, void *kern, void *hw, void **usr, int len)
 			return -1;
 		return 0;
 	}
-
 	return -1;
 }
 EXPORT_SYMBOL(maplib_lookup_usr);
@@ -272,6 +289,7 @@ static void maplib_tabs_free(struct maplib_priv *priv)
 		}
 	}
 }
+#endif
 
 static void maplib_free_blks(struct maplib_priv *priv)
 {
@@ -396,10 +414,12 @@ static int maplib_mmap(struct file *filp, struct vm_area_struct *vma)
 	/* Remember the User VMA that we have mapped */
 	priv->user_vma = vma;
 
+#if MAPLIB_KERN_TO_USR_LOOKUP
 	/* Now that we know the User-address and Kernel addresses,
 	 * the Kernel->UserAdress Table can be built.
 	 */
 	maplib_tabs_build(priv);
+#endif
 
 #ifdef MAPLIB_DEBUG
 	printk(KERN_DEBUG "MAPLIB: suceeded in mapping memory\n");
@@ -421,7 +441,9 @@ static void maplib_vma_close(struct vm_area_struct *area)
 	printk(KERN_DEBUG "MAPLIB: unmapping, telling drivers\n");
 #endif
 	maplib_drv_unreg_all(priv);
+#if MAPLIB_KERN_TO_USR_LOOKUP
 	maplib_tabs_free(priv);
+#endif
 
 	/* Mark Block as not mapped */
 	priv->user_vma = NULL;
@@ -521,7 +543,6 @@ static long maplib_ioctl(
 	unsigned long arg)
 {
 	struct maplib_priv *priv;
-	int err;
 	void __user *argp = (void __user *)arg;
 
 	priv = (struct maplib_priv *)filp->private_data;
@@ -532,15 +553,9 @@ static long maplib_ioctl(
 	}
 
 	/* Verify READ/WRITE Access to user provided buffer */
-	err = 0;
-	if (_IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
-
-	if ((!err) && (_IOC_DIR(cmd) & _IOC_READ))
-		err = !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
-
-	if (err)
-		return -EFAULT;
+	if (_IOC_DIR(cmd) & (_IOC_WRITE | _IOC_READ))
+		if (!access_ok((void __user *)arg, _IOC_SIZE(cmd)))
+			return -EFAULT;
 
 	switch (cmd) {
 	case MAPLIB_IOCTL_SETUP:
@@ -554,7 +569,7 @@ static long maplib_ioctl(
 	case MAPLIB_IOCTL_MMAPINFO:
 	{
 		struct maplib_mmap_info mmapinfo;
-		err = maplib_mmapinfo_get(priv, &mmapinfo);
+		int err = maplib_mmapinfo_get(priv, &mmapinfo);
 		if ( err )
 			return err;
 		if(copy_to_user(argp, &mmapinfo, sizeof(struct maplib_mmap_info)))
@@ -640,3 +655,4 @@ module_exit(maplib_exit);
 MODULE_AUTHOR("Cobham Gaisler AB.");
 MODULE_DESCRIPTION("GRLIB Help Memory MAP Library Driver");
 MODULE_LICENSE("GPL");
+MODULE_ALIAS("platform:grlib-maplib");
