@@ -1,7 +1,7 @@
 /*
  * Cobham Gaisler GRSPW2 SpaceWire Kernel Library Interface.
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,6 +34,7 @@
 #include <linux/workqueue.h>
 
 #include <linux/of_device.h>
+#include <linux/of_irq.h>
 #include <linux/of_platform.h>
 #include <linux/dma-mapping.h>
 
@@ -62,41 +63,41 @@
 #endif
 
 struct grspw_dma_regs {
-	volatile unsigned int ctrl;	/* DMA Channel Control */
-	volatile unsigned int rxmax;	/* RX Max Packet Length */
-	volatile unsigned int txdesc;	/* TX Descriptor Base/Current */
-	volatile unsigned int rxdesc;	/* RX Descriptor Base/Current */
-	volatile unsigned int addr;	/* Address Register */
-	volatile unsigned int resv[3];
+	volatile u32 ctrl;	/* DMA Channel Control */
+	volatile u32 rxmax;	/* RX Max Packet Length */
+	volatile u32 txdesc;	/* TX Descriptor Base/Current */
+	volatile u32 rxdesc;	/* RX Descriptor Base/Current */
+	volatile u32 addr;	/* Address Register */
+	volatile u32 resv[3];
 };
 
 struct grspw_regs {
-	volatile unsigned int ctrl;
-	volatile unsigned int status;
-	volatile unsigned int nodeaddr;
-	volatile unsigned int clkdiv;
-	volatile unsigned int destkey;
-	volatile unsigned int time;
-	volatile unsigned int timer;	/* Used only in GRSPW1 */
-	volatile unsigned int resv1;
+	volatile u32 ctrl;
+	volatile u32 status;
+	volatile u32 nodeaddr;
+	volatile u32 clkdiv;
+	volatile u32 destkey;
+	volatile u32 time;
+	volatile u32 timer;	/* Used only in GRSPW1 */
+	volatile u32 resv1;
 
 	/* DMA Registers, ctrl.NCH determines number of ports, 
 	 * up to 4 channels are supported
 	 */
 	struct grspw_dma_regs dma[4];
 
-	volatile unsigned int icctrl;
-	volatile unsigned int icrx;
-	volatile unsigned int icack;
-	volatile unsigned int ictimeout;
-	volatile unsigned int ictickomask;
-	volatile unsigned int icaamask;
-	volatile unsigned int icrlpresc;
-	volatile unsigned int icrlisr;
-	volatile unsigned int icrlintack;
-	volatile unsigned int resv2;
-	volatile unsigned int icisr;
-	volatile unsigned int resv3;
+	volatile u32 icctrl;
+	volatile u32 icrx;
+	volatile u32 icack;
+	volatile u32 ictimeout;
+	volatile u32 ictickomask;
+	volatile u32 icaamask;
+	volatile u32 icrlpresc;
+	volatile u32 icrlisr;
+	volatile u32 icrlintack;
+	volatile u32 resv2;
+	volatile u32 icisr;
+	volatile u32 resv3;
 };
 
 /* GRSPW - Control Register - 0x00 */
@@ -269,16 +270,16 @@ struct grspw_regs {
 
 /* RX Buffer Descriptor */
 struct grspw_rxbd {
-   volatile unsigned int ctrl;
-   volatile unsigned int addr;
+   volatile u32 ctrl;
+   volatile u32 addr;
 };
 
 /* TX Buffer Descriptor */
 struct grspw_txbd {
-   volatile unsigned int ctrl;
-   volatile unsigned int haddr;
-   volatile unsigned int dlen;
-   volatile unsigned int daddr;
+   volatile u32 ctrl;
+   volatile u32 haddr;
+   volatile u32 dlen;
+   volatile u32 daddr;
 };
 
 /* GRSPW - DMA RXBD Ctrl */
@@ -324,12 +325,28 @@ struct grspw_txbd {
 #define BDTAB_SIZE 0x400	/* BD Table Size (RX or TX) */
 #define BDTAB_ALIGN 0x400	/* BD Table Alignment Requirement */
 
-#ifdef CONFIG_SPARC_LEON
+#if (defined(CONFIG_SPARC_LEON) || defined(CONFIG_RISCV))
 /* Memory and HW Registers Access routines. All 32-bit access routines */
-#define BD_WRITE(addr, val) (*(volatile unsigned int *)(addr) = (unsigned int)(val))
-#define BD_READ(addr) (*(volatile unsigned long *)(addr))
-#define REG_WRITE(addr, val) (*(volatile unsigned int *)(addr) = (unsigned int)(val))
-#define REG_READ(addr) (*(volatile unsigned int *)(addr))
+static inline void grspw_write_bd(volatile u32 *bd, u32 val)
+{
+#if 0
+	__raw_writel(cpu_to_be32(val), bd);
+#else
+	__raw_writel(val, bd);
+#endif
+}
+static inline u32 grspw_read_bd(volatile u32 *bd)
+{
+#if 0
+	return be32_to_cpu(__raw_readl(bd));
+#else
+	return __raw_readl(bd);
+#endif
+}
+#define BD_WRITE(addr, val) (grspw_write_bd((volatile u32 *)(addr), (u32)(val)))
+#define BD_READ(addr) grspw_read_bd((volatile u32 *)(addr))
+#define REG_WRITE(addr, val) (*(volatile u32 *)(addr) = (u32)(val))
+#define REG_READ(addr) (*(volatile u32 *)(addr))
 #else
 #error UNSUPPORTED ARCHITECTURE
 #endif
@@ -360,7 +377,7 @@ struct grspw_rxring {
 
 struct grspw_dma_priv {
 	struct grspw_priv *core;	/* GRSPW Core */
-	struct grspw_dma_regs *regs;	/* DMA Channel Registers */
+	struct grspw_dma_regs __iomem *regs;	/* DMA Channel Registers */
 	int index;			/* DMA Channel Index @ GRSPW core */
 	int open;			/* DMA Channel opened by user */
 	int started;			/* DMA Channel activity (start|stop) */
@@ -379,8 +396,7 @@ struct grspw_dma_priv {
 
 	/* RX Descriptor Ring */
 	struct grspw_rxbd *rx_bds;		/* Descriptor Address */
-	struct grspw_rxbd *rx_bds_pa;		/* Descriptor Physical Address */
-	dma_addr_t rx_bds_base_phys;
+	dma_addr_t rx_bds_pa;			/* Descriptor Physical Address */
 	struct grspw_rxring *rx_ring_base;
 	struct grspw_rxring *rx_ring_head;	/* Next descriptor to enable */
 	struct grspw_rxring *rx_ring_tail;	/* Oldest enabled Descriptor */
@@ -410,7 +426,7 @@ struct grspw_dma_priv {
 
 	/* TX Descriptor Ring */
 	struct grspw_txbd *tx_bds;		/* Descriptor Address */
-	struct grspw_txbd *tx_bds_pa;		/* Descriptor Physical Address */
+	dma_addr_t tx_bds_pa;			/* Descriptor Physical Address */
 	struct grspw_txring *tx_ring_base;
 	struct grspw_txring *tx_ring_head;
 	struct grspw_txring *tx_ring_tail;
@@ -541,7 +557,7 @@ void *grspw_open(int dev_no)
 		priv->open = 0;
 		return NULL;
 		}
-	if ((unsigned int)priv->bd_mem & (BDTAB_ALIGN-1)) {
+	if ((long)priv->bd_mem & (BDTAB_ALIGN-1)) {
 		printk(KERN_INFO "GRSPW: Descriptors not aligned to boundary: %p\n",
 					priv->bd_mem);
 		dma_free_coherent(priv->dev, priv->bdtabsize, priv->bd_mem,
@@ -559,12 +575,10 @@ void *grspw_open(int dev_no)
 		priv->dma[i].open = 0;
 		priv->dma[i].rx_bds = (struct grspw_rxbd *)
 			(priv->bd_mem + i*BDTAB_SIZE*2);
-		priv->dma[i].rx_bds_pa = (struct grspw_rxbd *)
-			(priv->bd_mem_pa + BDTAB_SIZE*(2*i));
+		priv->dma[i].rx_bds_pa = priv->bd_mem_pa + BDTAB_SIZE*(2*i);
 		priv->dma[i].tx_bds = (struct grspw_txbd *)
 			(priv->bd_mem + BDTAB_SIZE*(2*i+1));
-		priv->dma[i].tx_bds_pa = (struct grspw_txbd *)
-			(priv->bd_mem_pa + BDTAB_SIZE*(2*i+1));
+		priv->dma[i].tx_bds_pa = priv->bd_mem_pa + BDTAB_SIZE*(2*i+1);
 	}
 
 	/* Basic initialization of hardware, clear some registers but
@@ -1510,7 +1524,6 @@ STATIC void grspw_dma_reset(struct grspw_dma_priv *dma)
 
 	grspw_dma_stats_clr(dma);
 }
-EXPORT_SYMBOL(grspw_dma_reset);
 
 int grspw_dma_close(void *c)
 {
@@ -1679,7 +1692,7 @@ void grspw_dma_tx_count(void *c, int *send, int *sched, int *sent, int *hw)
 		 */
 		hwbd = REG_READ(&dma->regs->txdesc);
 		tailbd = dma->tx_ring_tail->bd;
-		diff = ((hwbd - (unsigned int)tailbd) / GRSPW_TXBD_SIZE) &
+		diff = ((hwbd - (unsigned long)tailbd) / GRSPW_TXBD_SIZE) &
 			(GRSPW_TXBD_NR - 1);
 		/* Handle special case when HW and SW pointers are equal
 		 * because all TX descriptors have been processed by HW.
@@ -1935,7 +1948,7 @@ void grspw_dma_rx_count(void *c, int *ready, int *sched, int *recv, int *hw)
 		 */
 		hwbd = REG_READ(&dma->regs->rxdesc);
 		tailbd = dma->rx_ring_tail->bd;
-		diff = ((hwbd - (unsigned int)tailbd) / GRSPW_RXBD_SIZE) &
+		diff = ((hwbd - (unsigned long)tailbd) / GRSPW_RXBD_SIZE) &
 			(GRSPW_RXBD_NR - 1);
 		/* Handle special case when HW and SW pointers are equal
 		 * because all RX descriptors have been processed by HW.
@@ -2614,12 +2627,9 @@ void grspw_initialize_user(void *(*devfound)(int), void (*devremove)(int,void*))
 }
 EXPORT_SYMBOL(grspw_initialize_user);
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-static int __devinit grspw_of_probe(struct platform_device *of_dev, const struct of_device_id *match)
-#else
 static int grspw_of_probe(struct platform_device *of_dev)
-#endif
 {
+	struct device_node *onp = of_dev->dev.of_node;
 	struct grspw_priv *priv;
 	int i, size, err, len;
 	unsigned int ctrl;
@@ -2648,16 +2658,21 @@ static int grspw_of_probe(struct platform_device *of_dev)
 		return -ENOMEM;
 	memset(priv, 0, sizeof(*priv));
 
+
 	/* MAP Registers into Kernel Virtual Address Space */
-	priv->regs = of_ioremap(&of_dev->resource[0], 0,
-				 resource_size(&of_dev->resource[0]),
-				 "grlib-grspw regs");
-	if (priv->regs == NULL) {
-		err = -EIO;
-		goto error1;
+	priv->regs = devm_platform_ioremap_resource(of_dev, 0);
+	if (IS_ERR(priv->regs)) {
+		err = PTR_ERR(priv->regs);
+		goto error;
 	}
+
 	/* Get IRQ number of GRSPW device */
-	priv->irq = of_dev->archdata.irqs[0];
+	priv->irq = irq_of_parse_and_map(onp, 0);
+	if (!priv->irq) {
+		dev_err(&of_dev->dev, "no irq found\n");
+		err = -ENODEV;
+		goto error;
+	}
 	priv->dev = &of_dev->dev;
 
 	SPIN_INIT(&priv->devlock);
@@ -2683,13 +2698,9 @@ static int grspw_of_probe(struct platform_device *of_dev)
 	if ( pversion && (len == 4) )
 		priv->hwsup.hw_version = *pversion;
 	pdevice = 0;
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-	devid = (int)match->data;
-#else
 	pdevice = of_get_property(of_dev->dev.of_node, "device", &len);
 	if ( pdevice && (len == 4) )
 		devid = *pdevice;
-#endif
 	priv->hwsup.hw_version |= devid << 16;
 	if ( (devid == GAISLER_SPW2) || (devid == GAISLER_SPW2_DMA) ) {
 		priv->hwsup.strip_adr = 1; /* All GRSPW2 can strip Address */
@@ -2748,7 +2759,7 @@ static int grspw_of_probe(struct platform_device *of_dev)
 	if ( err ) {
 		printk(KERN_NOTICE "GRSPW[%d]: Failed installing IRQ %d\n",
 			priv->index, priv->irq);
-		goto error2;
+		goto error;
 	}
 
 	dev_set_drvdata(&of_dev->dev, priv);
@@ -2761,10 +2772,7 @@ static int grspw_of_probe(struct platform_device *of_dev)
 
 	return 0;
 
-error2:
-	of_iounmap(&of_dev->resource[0], priv->regs, resource_size(&of_dev->resource[0]));
-error1:
-
+error:
 	return err;
 }
 
@@ -2779,7 +2787,6 @@ static int grspw_of_remove(struct platform_device *of_dev)
 	free_irq(priv->irq, priv);
 
 	priv_tab[priv->index] = NULL;
-	of_iounmap(&of_dev->resource[0], priv->regs, resource_size(&of_dev->resource[0]));
 
 	kfree(priv);
 	dev_set_drvdata(&of_dev->dev, NULL);
@@ -2787,7 +2794,7 @@ static int grspw_of_remove(struct platform_device *of_dev)
 	return 0;
 }
 
-static struct of_device_id grspw_of_match[] = {
+static const struct of_device_id grspw_of_match[] = {
 #if 0 /* GRSPW1 not tested yet... */
 	{ /* SpaceWire 1 Interface */
 	 .name = "GAISLER_SPW",
@@ -2814,16 +2821,20 @@ static struct of_device_id grspw_of_match[] = {
 	 .name = "01_08a",
 	 .data = (void *)GAISLER_SPW2_DMA,
 	 },
+	{ /* SpaceWire 2 Interface */
+	 .compatible = "gaisler,grspw2",
+	 .data = (void *)GAISLER_SPW2,
+	 },
+	{ /* SpaceWire Router DMA Interface */
+	 .compatible = "gaisler,grspw2_dma",
+	 .data = (void *)GAISLER_SPW2_DMA,
+	 },
 	{}, /* Mark end */
 };
 
 MODULE_DEVICE_TABLE(of, grspw_of_match);
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-static struct of_platform_driver grspw_of_driver = {
-#else
 static struct platform_driver grspw_of_driver = {
-#endif
 	.driver = {
 		.name = "grlib-grspw",
 		.owner = THIS_MODULE,
@@ -2849,11 +2860,7 @@ static int __init grspw_init(void)
 	grspw_initialized = 1;
 
 	/* Register Platform Driver */
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-	return of_register_platform_driver(&grspw_of_driver);
-#else
 	return platform_driver_register(&grspw_of_driver);
-#endif
 }
 
 static void __exit grspw_exit (void)
@@ -2865,11 +2872,7 @@ static void __exit grspw_exit (void)
 	}
 	grspw_count = 0;
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-	of_unregister_platform_driver(&grspw_of_driver);
-#else
 	platform_driver_unregister(&grspw_of_driver);
-#endif
 }
 
 module_init(grspw_init);
@@ -2878,3 +2881,4 @@ module_exit(grspw_exit);
 MODULE_AUTHOR("Cobham Gaisler AB.");
 MODULE_DESCRIPTION("Cobham Gaisler GRSPW SpaceWire Library");
 MODULE_LICENSE("GPL");
+MODULE_ALIAS("platform:grlib-grspw");
