@@ -1,7 +1,7 @@
 /*
  * Cobham Gaisler GRSPW-ROUTER APB-Register access driver.
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,24 +35,24 @@
 #include <linux/grlib/grspw_router.h>
 #include <linux/grlib/devno.h>
 
-#define REG_READ(adr) (*(volatile unsigned int *)(adr))
-#define REG_WRITE(adr, value) (*(volatile unsigned int *)(adr) = (value))
+#define REG_READ(adr) (*(volatile u32 *)(adr))
+#define REG_WRITE(adr, value) (*(volatile u32 *)(adr) = (value))
 
 struct router_regs {
-	unsigned int resv1;		/* 0x000 */
-	unsigned int psetup[255];	/* 0x004 */
-	unsigned int resv2[32];		/* 0x400 */
-	unsigned int routes[224];	/* 0x480 */
-	unsigned int pctrl[32];		/* 0x800 */
-	unsigned int psts[32];		/* 0x880 */
-	unsigned int tprescaler;	/* 0x900 */
-	unsigned int treload[31];	/* 0x904 */
-	unsigned int resv3[32];		/* 0x980 */
-	unsigned int cfgsts;		/* 0xA00 */
-	unsigned int timecode;		/* 0xA04 */
-	unsigned int ver;		/* 0xA08 */
-	unsigned int idiv;		/* 0xA0C */
-	unsigned int cfgwe;		/* 0xA10 */
+	u32 resv1;		/* 0x000 */
+	u32 psetup[255];	/* 0x004 */
+	u32 resv2[32];		/* 0x400 */
+	u32 routes[224];	/* 0x480 */
+	u32 pctrl[32];		/* 0x800 */
+	u32 psts[32];		/* 0x880 */
+	u32 tprescaler;		/* 0x900 */
+	u32 treload[31];	/* 0x904 */
+	u32 resv3[32];		/* 0x980 */
+	u32 cfgsts;		/* 0xA00 */
+	u32 timecode;		/* 0xA04 */
+	u32 ver;		/* 0xA08 */
+	u32 idiv;		/* 0xA0C */
+	u32 cfgwe;		/* 0xA10 */
 };
 
 struct router_priv {
@@ -264,7 +264,6 @@ int router_tc_read(struct router_priv *priv, unsigned int *tc)
 static long router_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 {
 	struct router_priv *priv;
-	int err;
 	void __user *argp = (void __user *)arg;
 
 	priv = (struct router_priv *)filp->private_data;
@@ -275,15 +274,9 @@ static long router_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	}
 
 	/* Verify READ/WRITE Access to user provided buffer */
-	err = 0;
-	if (_IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
-
-	if ((!err) && (_IOC_DIR(cmd) & _IOC_READ))
-		err = !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
-
-	if (err)
-		return -EFAULT;
+	if (_IOC_DIR(cmd) & (_IOC_WRITE | _IOC_READ))
+		if (!access_ok((void __user *)arg, _IOC_SIZE(cmd)))
+			return -EFAULT;
 
 	switch (cmd) {
 
@@ -356,7 +349,7 @@ static long router_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	/* Set configuration write enable */
 	case GRSPWR_IOCTL_WE_SET:
 	{
-		return router_we_set(priv, (int)argp);
+		return router_we_set(priv, (long)argp);
 	}
 
 	/* Set/Get Port Control/Status */
@@ -376,7 +369,7 @@ static long router_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	/* Set Router Configuration/Status Register */
 	case GRSPWR_IOCTL_CFGSTS_SET:
 	{
-		return router_cfgsts_set(priv, (int)argp);
+		return router_cfgsts_set(priv, (long)argp);
 	}
 
 	/* Get Router Configuration/Status Register */
@@ -405,11 +398,7 @@ static long router_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	return 0;
 }
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-static int __devinit router_of_probe(struct platform_device *of_dev, const struct of_device_id *match)
-#else
 static int router_of_probe(struct platform_device *of_dev)
-#endif
 {
 	struct router_priv *priv;
 	int result, err, len;
@@ -429,12 +418,11 @@ static int router_of_probe(struct platform_device *of_dev)
 
 	memset(priv, 0, sizeof(*priv));
 
-	priv->regs = of_ioremap(&of_dev->resource[0], 0,
-				 resource_size(&of_dev->resource[0]),
-				 "grlib-grspw-router regs");
-	if (priv->regs == NULL) {
-		err = -EIO;
-		goto error1;
+	/* MAP Registers into Kernel Virtual Address Space */
+	priv->regs = devm_platform_ioremap_resource(of_dev, 0);
+	if (IS_ERR(priv->regs)) {
+		err = PTR_ERR(priv->regs);
+		goto error;
 	}
 
 	/* Register character device in registered region */
@@ -445,7 +433,7 @@ static int router_of_probe(struct platform_device *of_dev)
 	if ( result ) {
 		printk(KERN_ERR "GRSPW_ROUTER: Failed adding CHAR dev\n");
 		err = -ENODEV;
-		goto error2;
+		goto error;
 	}
 
 	router_hwinfo(priv, &priv->hwinfo);
@@ -466,10 +454,7 @@ static int router_of_probe(struct platform_device *of_dev)
 
 	return 0;
 
-error2:
-	of_iounmap(&of_dev->resource[0], priv->regs, resource_size(&of_dev->resource[0]));
-error1:
-
+error:
 	return err;
 }
 
@@ -479,31 +464,28 @@ static int router_of_remove(struct platform_device *of_dev)
 
 	cdev_del(&priv->cdev);
 
-	of_iounmap(&of_dev->resource[0], priv->regs, resource_size(&of_dev->resource[0]));
-
 	kfree(priv);
 	dev_set_drvdata(&of_dev->dev, NULL);
 
 	return 0;
 }
 
-static struct of_device_id router_of_match[] = {
+static const struct of_device_id router_of_match[] = {
 	{
 	 .name = "GAISLER_SPWROUTER",
 	 },
 	{
 	 .name = "01_08b",
 	 },
+	{
+	 .compatible = "gaisler,spwrtr",
+	 },
 	{},
 };
 
 MODULE_DEVICE_TABLE(of, router_of_match);
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-static struct of_platform_driver router_of_driver = {
-#else
 static struct platform_driver router_of_driver = {
-#endif
 	.driver = {
 		.name = "grlib-spwrouter",
 		.owner = THIS_MODULE,
@@ -528,11 +510,7 @@ static int __init grspw_router_init(void)
 
 	router_count = 0;
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-	return of_register_platform_driver(&router_of_driver);
-#else
 	return platform_driver_register(&router_of_driver);
-#endif
 }
 
 static void __exit grspw_router_cleanup(void)
@@ -543,11 +521,7 @@ static void __exit grspw_router_cleanup(void)
 	dev_id = MKDEV(SPWROUTER_MAJOR, SPWROUTER_MINOR);
 	unregister_chrdev_region(dev_id, SPWROUTER_DEVCNT);
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 36)
-	of_unregister_platform_driver(&router_of_driver);
-#else
 	platform_driver_unregister(&router_of_driver);
-#endif
 }
 
 module_init(grspw_router_init);
@@ -556,3 +530,4 @@ module_exit(grspw_router_cleanup);
 MODULE_AUTHOR("Cobham Gaisler AB.");
 MODULE_DESCRIPTION("Cobham Gaisler SpaceWire Router APB Driver");
 MODULE_LICENSE("GPL");
+MODULE_ALIAS("platform:grlib-grspwrouter");
