@@ -1,7 +1,7 @@
 /*
  * Cobham Gaisler GRSPW user space access Driver.
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -52,10 +52,11 @@
 #define init_MUTEX_LOCKED(a) sema_init(a, 0);
 #endif
 
-/* Which Memory MAP device node that should be used. Currently hardcoded.
- * should be configurable per GRSPW Device during runtime.
+/* Which default Memory MAP device node that should be used. User can
+ * config maplib pool index per GRSPW Device during runtime using
+ * GRSPW_IOCTL_BUFCFG ioctl command.
  */
-#define MMAPLIB_POOL 0
+#define MMAPLIB_POOL_DEFAULT 0
 
 /* Do not enable Debug */
 #undef GRSPWU_DEBUG
@@ -101,6 +102,7 @@ struct grspw_bufcfg grspwu_def_bufcfg =
 {
 	.rx_pkts = 1024,
 	.tx_pkts = 1024,
+	.maplib_pool_idx = MMAPLIB_POOL_DEFAULT,
 };
 
 static struct grspwu_priv *privu_tab[GRSPW_MAX];
@@ -108,8 +110,8 @@ static struct grspwu_priv *privu_tab[GRSPW_MAX];
 static int grspwu_open(struct inode *inode, struct file *file);
 static int grspwu_release(struct inode *inode, struct file *file);
 static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
-static int grspwu_read(struct file *filp, char __user *buff, size_t count, loff_t *offp);
-static int grspwu_write(struct file *filp, const char __user *buff, size_t count, loff_t *offp);
+static ssize_t grspwu_read(struct file *filp, char __user *buff, size_t count, loff_t *offp);
+static ssize_t grspwu_write(struct file *filp, const char __user *buff, size_t count, loff_t *offp);
 
 
 struct file_operations grspwu_fops = {
@@ -124,7 +126,7 @@ struct file_operations grspwu_fops = {
 };
 
 void grspwu_stop(struct grspwu_priv *priv);
-void grspwu_unmap(int idx, void *priv);
+void grspwu_unmap(int idx, void *p);
 
 static int grspwu_open(struct inode *inode, struct file *file)
 {
@@ -221,7 +223,8 @@ int grspwu_bufcfg(struct grspwu_priv *priv, struct grspw_bufcfg *bufcfg)
 	 */
 	if ( (priv->bufcfg.rx_pkts != bufcfg->rx_pkts) ||
 	     (priv->bufcfg.tx_pkts != bufcfg->tx_pkts) ) {
-		memcpy(&priv->bufcfg, bufcfg, sizeof(*bufcfg));
+		priv->bufcfg.rx_pkts = bufcfg->rx_pkts;
+		priv->bufcfg.tx_pkts = bufcfg->tx_pkts;
 		if ( priv->_pkt_tx_structs ) {
 			kfree(priv->_pkt_tx_structs);
 			priv->_pkt_tx_structs = NULL;
@@ -234,6 +237,11 @@ int grspwu_bufcfg(struct grspwu_priv *priv, struct grspw_bufcfg *bufcfg)
 		}
 	}
 
+	/* Basic sanity check of the input argument */
+	if (maplib_drv_chkvalid(bufcfg->maplib_pool_idx))
+		return -EINVAL;
+	priv->bufcfg.maplib_pool_idx = bufcfg->maplib_pool_idx;
+
 	return 0;
 }
 
@@ -497,8 +505,8 @@ int grspwu_start(struct grspwu_priv *priv, int options)
 		pktbase[i].reserved = 0;
 		pktbase[i].hlen = 0;
 		pktbase[i].dlen = 0;
-		pktbase[i].data = NULL;
-		pktbase[i].hdr = NULL;
+		pktbase[i].data = GRSPW_PKT_DMAADDR_INVALID;
+		pktbase[i].hdr = GRSPW_PKT_DMAADDR_INVALID;
 	}
 	/* Initialize TX Packet structure List */
 	priv->tx_pktstrs.head = &pktbase[0];
@@ -512,8 +520,8 @@ int grspwu_start(struct grspwu_priv *priv, int options)
 		pktbase[i].reserved = 0;
 		pktbase[i].hlen = 0;
 		pktbase[i].dlen = 0;
-		pktbase[i].data = NULL;
-		pktbase[i].hdr = NULL;
+		pktbase[i].data = GRSPW_PKT_DMAADDR_INVALID;
+		pktbase[i].hdr = GRSPW_PKT_DMAADDR_INVALID;
 	}
 	/* Initialize RX Packet structure List */
 	priv->rx_pktstrs.head = &pktbase[0];
@@ -538,7 +546,8 @@ int grspwu_start(struct grspwu_priv *priv, int options)
 	priv->mmapdrv.name = priv->name;
 	priv->mmapdrv.unmap = grspwu_unmap;
 	priv->mmapdrv.next = NULL;
-	if ( maplib_drv_reg(MMAPLIB_POOL, &priv->mmapdrv) ) {
+	if ( maplib_drv_reg(priv->bufcfg.maplib_pool_idx,
+			    &priv->mmapdrv) ) {
 		/* Failed to register, this is because user has not setup
 		 * or mapped all memory to user space. We cannot proceed
 		 * until user has fixed this.
@@ -562,7 +571,7 @@ int grspwu_start(struct grspwu_priv *priv, int options)
 
 err_out:
 	/* Remove MMAP callback */
-	maplib_drv_unreg(MMAPLIB_POOL, &priv->mmapdrv);
+	maplib_drv_unreg(priv->bufcfg.maplib_pool_idx, &priv->mmapdrv);
 
 	/* Something bad happened, stop all opened DMA channels */
 	for (i=0; i<priv->chan_cnt; i++) {
@@ -595,12 +604,14 @@ void grspwu_stop(struct grspwu_priv *priv)
 	 * memory, user may still access the memory, however this driver
 	 * or hardware will not
 	 */
-	maplib_drv_unreg(MMAPLIB_POOL, &priv->mmapdrv);
+	maplib_drv_unreg(priv->bufcfg.maplib_pool_idx, &priv->mmapdrv);
 }
 
-void grspwu_unmap(int idx, void *priv)
+void grspwu_unmap(int idx, void *p)
 {
-	if ( idx != MMAPLIB_POOL )
+	struct grspwu_priv *priv = (struct grspwu_priv *)p;
+
+	if (idx != priv->bufcfg.maplib_pool_idx)
 		return;
 
 	/* User has done something bad: unmapped memory without stopping
@@ -661,7 +672,6 @@ int grspwu_tx_wait(struct grspwu_priv *priv, struct grspw_tx_wait_chan *wait)
 static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 {
 	struct grspwu_priv *priv;
-	int err;
 	void __user *argp = (void __user *)arg;
 
 	priv = (struct grspwu_priv *)filp->private_data;
@@ -672,15 +682,9 @@ static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	}
 
 	/* Verify READ/WRITE Access to user provided buffer */
-	err = 0;
-	if (_IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
-
-	if ((!err) && (_IOC_DIR(cmd) & _IOC_READ))
-		err = !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
-
-	if (err)
-		return -EFAULT;
+	if (_IOC_DIR(cmd) & (_IOC_WRITE | _IOC_READ))
+		if (!access_ok((void __user *)arg, _IOC_SIZE(cmd)))
+			return -EFAULT;
 
 	switch (cmd) {
 
@@ -753,7 +757,7 @@ static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 
 	case GRSPW_IOCTL_PORTCTRL:
 	{
-		return grspwu_port_ctrl(priv, (int)argp);
+		return grspwu_port_ctrl(priv, (long)argp);
 	}
 
 	case GRSPW_IOCTL_LINKSTATE:
@@ -767,7 +771,7 @@ static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 
 	case GRSPW_IOCTL_TC_SEND:
 	{
-		grspwu_tc_send(priv, (unsigned int)argp);
+		grspwu_tc_send(priv, (unsigned long)argp);
 		break;
 	}
 
@@ -811,7 +815,7 @@ static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 
 	case GRSPW_IOCTL_START:
 	{
-		return grspwu_start(priv, (int)argp);
+		return grspwu_start(priv, (long)argp);
 	}
 
 	case GRSPW_IOCTL_STOP:
@@ -991,7 +995,8 @@ int grspwu_recv(
 		pkts[pkts_cnt].dma_chan = dmachan;
 		pkts[pkts_cnt].resv1 = 0;
 		pkts[pkts_cnt].dlen = pkt->dlen;
-		pkts[pkts_cnt].data = pkt->hdr; /* UserSpaceAdr temporarily stored in HDR field */
+		/* UserSpaceAdr temporarily stored in HDR field */
+		pkts[pkts_cnt].data = (void *)((unsigned long)pkt->hdr);
 		pkts[pkts_cnt].pkt_id = pkt->pkt_id;
 #ifdef GRSPWU_DEBUG
 		if ( (pkts[pkts_cnt].flags | pkt->flags) & 0x0900 ) {
@@ -1044,6 +1049,7 @@ int grspwu_send(struct grspwu_priv *priv, int dmachan,
 	int pktidx, count, size, left;
 	struct grspw_wtxpkt pkts[32];
 	int retval = 0;
+	phys_addr_t pa;
 #ifdef GRSPWU_DEBUG
 	int tmp, tmp2;
 #endif
@@ -1106,11 +1112,11 @@ int grspwu_send(struct grspwu_priv *priv, int dmachan,
 		pkt->hlen = pkts[pktidx].hlen;
 		pkt->dlen = pkts[pktidx].dlen;
 		if ( (pkt->dlen > 0) && maplib_lookup_kern(
-				MMAPLIB_POOL,
+				priv->bufcfg.maplib_pool_idx,
 				pkts[pktidx].data,
 				NULL, /* Kernel address Unused */
-				&pkt->data, /* hardware address used in DMA */
-				pkt->dlen) ) {
+				&pa, /* hardware address used in DMA */
+				pkt->dlen)) {
 			/* Address translation Failed, memory may go over a 
 			 * block boundary, or comletely outside. Bug in USER
 			 * packet handling, or an attempt to destory kernel
@@ -1121,11 +1127,15 @@ int grspwu_send(struct grspwu_priv *priv, int dmachan,
 			retval = -EPERM;
 			goto out;
 		}
+		/* Truncate: Currently GRSPW2 IPs are only capable of performing DMA to
+		 * lower 32-bit address space
+		 */
+		pkt->data = (u32)(pa & 0xFFFFFFFFUL);
 		if ( (pkt->hlen > 0) && maplib_lookup_kern(
-				MMAPLIB_POOL,
+				priv->bufcfg.maplib_pool_idx,
 				pkts[pktidx].hdr,
 				NULL, /* Kernel address unused */
-				&pkt->hdr, /* hardware address used in DMA */
+				&pa, /* hardware address used in DMA */
 				pkt->hlen) ) {
 			/* Address translation Failed, memory may go over a 
 			 * block boundary, or comletely outside. Bug in USER
@@ -1137,6 +1147,7 @@ int grspwu_send(struct grspwu_priv *priv, int dmachan,
 			 retval = -EPERM;
 			 goto out;
 		}
+		pkt->hdr = (u32)(pa & 0xFFFFFFFFUL);
 		pkt->pkt_id = pkts[pktidx].pkt_id;
 
 		pktidx++;
@@ -1184,6 +1195,7 @@ int grspwu_prepare(struct grspwu_priv *priv, int dmachan,
 	int pktidx, count, size, left;
 	struct grspw_wrxpkt pkts[64];
 	int retval = 0;
+	phys_addr_t pa;
 #ifdef GRSPWU_DEBUG
 	int tmp;
 #endif
@@ -1242,10 +1254,10 @@ int grspwu_prepare(struct grspwu_priv *priv, int dmachan,
 		pkt->hlen = 0;
 		pkt->dlen = 0; /* Will be overwritten */
 		if ( maplib_lookup_kern(
-				MMAPLIB_POOL,
+				priv->bufcfg.maplib_pool_idx,
 				pkts[pktidx].data,
 				NULL, /* Kernel Address unused */
-				&pkt->data, /* hardware address used in DMA */
+				&pa, /* hardware address used in DMA */
 				priv->dma_rxmaxlen[dmachan]) ) {
 			/* Address translation Failed, memory may go over a 
 			 * block boundary, or comletely outside. Bug in USER
@@ -1257,10 +1269,11 @@ int grspwu_prepare(struct grspwu_priv *priv, int dmachan,
 			 retval = -EPERM;
 			 goto out;
 		}
+		pkt->data = (u32)(pa & 0xFFFFFFFFUL);
 		/* Remember UserSpaceAdr in unused Header Address to avoid one
 		 * extra address translation in recv() function.
 		 */
-		pkt->hdr = pkts[pktidx].data;
+		pkt->hdr = (u32)((unsigned long)pkts[pktidx].data & 0xFFFFFFFFUL);
 		pkt->pkt_id = pkts[pktidx].pkt_id;
 
 		pktidx++;
@@ -1300,7 +1313,7 @@ out:
 	return retval;
 }
 
-static int grspwu_read(struct file *filp, char __user *buff, size_t count, loff_t *offp)
+static ssize_t grspwu_read(struct file *filp, char __user *buff, size_t count, loff_t *offp)
 {
 	struct grspwu_priv *priv;
 	int operation, chan_mask;
@@ -1308,7 +1321,7 @@ static int grspwu_read(struct file *filp, char __user *buff, size_t count, loff_
 
 	priv = filp->private_data;
 
-	if ( (unsigned int)buff & 0x3 )
+	if ( (unsigned long)buff & 0x3 )
 		return -EINVAL;
 
 	if ( priv->state == STATE_STOPPED )
@@ -1391,7 +1404,7 @@ static int grspwu_read(struct file *filp, char __user *buff, size_t count, loff_
 	}
 }
 
-static int grspwu_write(struct file *filp, const char __user *buff, size_t count, loff_t *offp)
+static ssize_t grspwu_write(struct file *filp, const char __user *buff, size_t count, loff_t *offp)
 {
 	struct grspwu_priv *priv;
 	int operation, chan, cnt;
@@ -1402,7 +1415,7 @@ static int grspwu_write(struct file *filp, const char __user *buff, size_t count
 	if ( priv->state < STATE_STARTED )
 		return -EBUSY;
 
-	if ( (unsigned int)buff & 0x3 )
+	if ( (unsigned long)buff & 0x3 )
 		return -EINVAL;
 
 	/* Get Operation from MSB of Length */
@@ -1525,3 +1538,4 @@ module_exit(grspwu_exit);
 MODULE_AUTHOR("Cobham Gaisler AB.");
 MODULE_DESCRIPTION("Cobham Gaisler GRSPW SpaceWire Driver");
 MODULE_LICENSE("GPL");
+MODULE_ALIAS("platform:grlib-grspwu");
