@@ -1,7 +1,7 @@
 /*
  * MAPLIB user space interface
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -83,7 +83,7 @@ int mmaplib_init(int idx, int blk_size, int blk_cnt)
 	/* Memory MAP driver's Buffers READ-and-WRITE  */
 	adr = mmap(NULL, mapi->buf_length, PROT_READ|PROT_WRITE, MAP_SHARED,
 			map_fd, start);
-	if ( (unsigned int)adr == 0xffffffff ) {
+	if ( adr == MAP_FAILED ) {
 		printf("MMAP Bufs Failed: %p, errno %d, %x\n", adr, errno, mapi->buf_length);
 		return -4;
 	}
