@@ -1,7 +1,7 @@
 /*
  * SpaceWire packet pool allocating library software
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -102,7 +102,7 @@ struct spwlib_poolset *spwlib_poolset_alloc(struct spwlib_pool_cfg *cfg)
 	pool = &poolset->pools[0];
 	/* Make packet start after the packet pool structs */
 	pktbase = (struct spwlib_pkt *)
-	      ((unsigned int)pool + pool_cnt*sizeof(struct spwlib_pool));
+	      ((unsigned long)pool + pool_cnt*sizeof(struct spwlib_pool));
 	for (i=0; i<poolset->pool_cnt; i++) {
 		pool->cfg = *desc;
 		pool->index = i;
