@@ -1,7 +1,7 @@
 /*
  * GRSPW DMA SpaceWire packet demonstration software
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <string.h>
+#include <arpa/inet.h>
 
 #include <linux/grlib/grspw.h>
 #include <linux/grlib/grspw_user.h>
@@ -347,6 +348,7 @@ int grspw_init(int idx)
 	/* Buffer Configuration */
 	bufcfg.rx_pkts = RXPKTS;
 	bufcfg.tx_pkts = TXPKTS;
+	bufcfg.maplib_pool_idx = MMAPLIB_POOL;
 	if ( grspw_bufcfg_set(dev, &bufcfg) ) {
 		grspw_close(dev);
 		return -1;
@@ -474,7 +476,7 @@ try:
 			printf("ERROR: map[%d] referenced\n", *map_alloc_idx);
 			return NULL;
 		}
-		retval = (void *)((unsigned int)map->start + *map_alloc_ofs);
+		retval = (void *)((unsigned long)map->start + *map_alloc_ofs);
 		(*map_alloc_ofs) += length;
 	}
 	/*printf("ALLOCED %s: %p %d\n", hdr ? "hdr" : "data", retval, length);*/
@@ -614,12 +616,12 @@ int pkts_init(struct grspw_device *device, int initiator)
 			pkt_hdr->protid = SPW_PROT_ID;
 			pkt_hdr->resv = 0;
 			/* Set PKT-ID to index */
-			pkt_hdr->pkt_number = pkt_number;
+			pkt_hdr->pkt_number = htonl(pkt_number);
 			pkts_content_tab[pkt_number] = pkt_content;
 
 			/* Init Data */
 			for (i=0; i<PKT_DATA_SIZE/2; i++) {
-				pkt_hdr->data[i] = pkt_content++;
+				pkt_hdr->data[i] = htons(pkt_content++);
 			}
 
 			/* Init Header and DST address in data */
@@ -705,7 +707,12 @@ void pktlist_set_flags(struct spwlib_list *lst, unsigned short flags)
 int debug_test(int got, int exp, void *adr)
 {
 	static int i = 0;
+#ifdef __sparc
 	asm volatile("ta 0x01\n\t"::);
+#else
+	/* TODO, what break instruction to use with RISCV? ebreak? */
+	while (1) ;
+#endif
 	return i++;
 }
 
@@ -727,16 +734,16 @@ int pktlist_check_rx(struct grspw_device *device, struct spwlib_list *lst, int c
 
 		/* Check Packet number (Sequence check) */
 		pkt_hdr = (struct pkt_hdr *)pkt->data;
-		if ( pkt_hdr->pkt_number != device->rxpkt_number_next ) {
+		if ( ntohl(pkt_hdr->pkt_number) != device->rxpkt_number_next ) {
 			printf("PKT RX SEQ ERROR: exp %d, got %d\n",
-				device->rxpkt_number_next, pkt_hdr->pkt_number);
+				device->rxpkt_number_next, ntohl(pkt_hdr->pkt_number));
 			return -2;
 		}
 		/* Check Packet Content */
 		if ( content ) {
-			exp = pkts_content_tab[pkt_hdr->pkt_number];
+			exp = pkts_content_tab[ntohl(pkt_hdr->pkt_number)];
 			for (i=0; i<PKT_DATA_SIZE/2; i++) {
-				got = pkt_hdr->data[i];
+				got = ntohs(pkt_hdr->data[i]);
 				if ( exp != got ) {
 					debug_test(got, exp, &pkt_hdr->data[i]);
 					printf(" BAD RX DATA in PACKET: got 0x%04x expected 0x%04x (%p)\n",
@@ -767,17 +774,17 @@ int pktlist_check_tx(struct grspw_device *device, struct spwlib_list *lst, int c
 	while ( pkt ) {
 		/* Check Packet number (Sequence check) */
 		pkt_hdr = (struct pkt_hdr *)pkt->data;
-		if ( pkt_hdr->pkt_number != device->txpkt_number_next ) {
+		if ( ntohl(pkt_hdr->pkt_number) != device->txpkt_number_next ) {
 			printf("PKT TX SEQ ERROR: exp %d, got %d\n",
-				device->txpkt_number_next, pkt_hdr->pkt_number);
+				device->txpkt_number_next, ntohl(pkt_hdr->pkt_number));
 			return -2;
 		}
 
 		/* Check Packet Content */
 		if ( content ) {
-			exp = pkts_content_tab[pkt_hdr->pkt_number];
+			exp = pkts_content_tab[ntohl(pkt_hdr->pkt_number)];
 			for (i=0; i<PKT_DATA_SIZE/2; i++) {
-				got = pkt_hdr->data[i];
+				got = ntohs(pkt_hdr->data[i]);
 				if ( exp != got ) {
 					debug_test(got, exp, &pkt_hdr->data[i]);
 					printf(" BAD TX DATA in PACKET: got 0x%04x expected 0x%04x (%p)\n",
@@ -1175,9 +1182,9 @@ int main(int argc, char *argv[])
 		grspw_stats_print(&devs[i].dev, &stats);
 	}
 	printf(" t0.tv_sec:     %u\n", (unsigned int)t0.tv_sec);
-	printf(" t0.tv_usec:    %d\n", t0.tv_usec);
+	printf(" t0.tv_usec:    %ld\n", (long)t0.tv_usec);
 	printf(" t1.tv_sec:     %u\n", (unsigned int)t1.tv_sec);
-	printf(" t1.tv_usec:    %d\n", t1.tv_usec);
+	printf(" t1.tv_usec:    %ld\n", (long)t1.tv_usec);
 	time = (float)t1.tv_sec + (float)t1.tv_usec/1000000;
 	time = time - (float)t0.tv_sec - (float)t0.tv_usec/1000000;
 	printf(" Time:          %f seconds\n", time);
@@ -1194,7 +1201,7 @@ int main(int argc, char *argv[])
 	if ( spwlib_list_is_empty(&devs[SPW_INITIATOR].check_list) == 0 ) {
 		struct pkt_hdr *pkt_hdr = (struct pkt_hdr *)
 			devs[SPW_INITIATOR].check_list.head->data;
-		devs[SPW_INITIATOR].rxpkt_number_next = pkt_hdr->pkt_number;
+		devs[SPW_INITIATOR].rxpkt_number_next = ntohl(pkt_hdr->pkt_number);
 
 		result = pktlist_check_rx(&devs[SPW_INITIATOR],
 					&devs[SPW_INITIATOR].check_list,
