@@ -1,7 +1,7 @@
 /*
  * Test MAPLIB driver
  *
- * Copyright (c) 2016 Cobham Gaisler AB
+ * Copyright (c) 2016-2022 Cobham Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -103,7 +103,7 @@ int mmap_init(int idx, int blk_size, int blk_cnt)
 	/* Memory MAP driver's Buffers READ-and-WRITE  */
 	adr = mmap(NULL, mapi->buf_length, PROT_READ|PROT_WRITE, MAP_SHARED,
 			map_fd, start);
-	if ( (unsigned int)adr == 0xffffffff ) {
+	if ( adr == MAP_FAILED ) {
 		printf("MMAP Bufs Failed: %p, errno %d, %x\n", adr, errno, mapi->buf_length);
 		return -1;
 	}
@@ -119,7 +119,7 @@ int mmap_init(int idx, int blk_size, int blk_cnt)
 		maps[i].start = adr;
 		maps[i].length = len;
 		
-		printf("MAPPED OFS=%x, BASE=%p, LEN=%x\n", maps[i].ofs, maps[i].start, maps[i].length);
+		printf("MAPPED OFS=%x, BASE=%p, LEN=%lx\n", maps[i].ofs, maps[i].start, maps[i].length);
 		fflush(NULL);
 		i++;
 
